import logging
import os
import time
import sys
if sys.version < '3.8':
  import inspect

# Used by the application to determine global logging level. Specific levels can be set with parameters. Can be passed as a parameter to siwim_i.
settings = dict()
global_settings = dict()
global_settings['folder'] = 'log'
global_settings['level'] = logging.INFO
global_settings['console_level'] = logging.INFO


def init_logger(name, folder=None, fmt=None, level=None, to_console=False, console_level=None, log_to=None):
  """This method is used to initialise the logger we're using for the current file.
  :param name: Name of the logger. Should be the name of the module or any other unique identifier.
  :param folder: Name of the folder in which to save logs relative to the root script.
  :param fmt: The format of messages logged to files.
  :param level: The level from which to log (for example, level of WARNING won't log levels DEBUG and INFO).
  :param to_console: Whether or not to write the output of the logger to console.
  :param console_level: The level from which to log to console.
  :param log_to: Specify if the file we want to log to does not have the same name as the logger (for example multiple loggers logging to the same file).
  """
  # As each logger may have it's own settings, each has it's own dict.
  settings[name] = dict()
  settings[name]['folder'] = global_settings['folder'] if folder is None else folder
  if sys.version < '3.8':  # Using a functionality from 3.8 to display the proper file
    settings[name]['format'] = '%(asctime)s %(levelname)-8s %(message)s' if fmt is None else fmt
  else:
    settings[name]['format'] = '%(asctime)s %(levelname)-8s [%(filename)s: %(lineno)d] %(message)s' if fmt is None else fmt
  settings[name]['level'] = global_settings['level'] if level is None else level
  settings[name]['console_level'] = global_settings['console_level'] if console_level is None else console_level
  settings[name]['console_formatter'] = '%(name)-9s %(asctime)s %(levelname)-8s %(message)s'
  settings[name]['log_to'] = name if log_to is None else log_to
  settings[name]['current_date'] = time.strftime('%Y-%m-%d')
  if not os.path.exists(settings[name]['folder']):
    os.mkdir(settings[name]['folder'])

  if to_console:
    console = logging.StreamHandler()
    console.setLevel(settings[name]['console_level'])
    console.setFormatter(logging.Formatter(fmt=settings[name]['console_formatter'], datefmt='%Y-%m-%d %H:%M:%S'))
    logging.getLogger(name).addHandler(console)

  logger = logging.getLogger(name)
  handler = logging.FileHandler('{0}/{1}_{2}.log'.format(settings[name]['folder'], settings[name]['current_date'], settings[name]['log_to']), delay=True)
  handler.setFormatter(logging.Formatter(fmt=settings[name]['format'], datefmt='%H:%M:%S'))
  logger.addHandler(handler)
  logger.setLevel(settings[name]['level'])

  # To avoid having to figure out which method to call in log, we store these mappings
  settings[name]['severities'] = {logging.DEBUG: logger.debug, logging.INFO: logger.info, logging.WARNING: logger.warning, logging.ERROR: logger.error, logging.CRITICAL: logger.critical}
  return logger


def configure_logger(logger, folder='log', level=logging.INFO, console=logging.INFO):
  """Used for setting logging parameters.
  :param logger: The logger which we're configuring.
  :param folder: Sets folder relative to calling script.
  :param level: Sets file debugging level.
  :param console: Sets console debugging level.
  """
  settings[logger.name]['folder'] = folder
  settings[logger.name]['level'] = level
  settings[logger.name]['console_level'] = console


def configure_default_logger(folder='log', level=logging.INFO, console=logging.INFO):
  """Used for setting global logging parameters.
  :param folder: Sets folder relative to calling script.
  :param level: Sets file debugging level.
  :param console: Sets console debugging level.
  """
  global_settings['folder'] = folder
  global_settings['level'] = level
  global_settings['console_level'] = console


def check_handler(logger):
  # When date changes, we have to change the file handler of the logger.
  if time.strftime('%Y-%m-%d') != settings[logger.name]['current_date']:
    settings[logger.name]['current_date'] = time.strftime('%Y-%m-%d')
    for handler in logger.handlers[:]:
      if isinstance(handler, logging.FileHandler):  # We only want to remove the FileHandler ... StreamHandlers are fine
        logger.removeHandler(handler)
    handler = logging.FileHandler('{0}/{1}_{2}.log'.format(settings[logger.name]['folder'], settings[logger.name]['current_date'], settings[logger.name]['log_to']), delay=True)
    handler.setFormatter(logging.Formatter(fmt=settings[logger.name]['format'], datefmt='%H:%M:%S'))
    logger.addHandler(handler)


def log(logger, msg, sev=logging.INFO, exc_info=False):
  check_handler(logger)
  if sys.version < '3.8':
    settings[logger.name]['severities'][sev]('{0} [{1}: {2}]'.format(msg, inspect.stack()[1][1].split('\\')[-1], inspect.stack()[1][2]), exc_info=exc_info)
  else:
    settings[logger.name]['severities'][sev](msg, exc_info=exc_info, stacklevel=2)  # Otherwise the displayed file/line number is of this file, which is rather useless.


def log_exception(logger, msg):
  # Convenience function; result is the same as log(logger, msg, sev=logging.CRITICAL, exc_info=True)
  check_handler(logger)
  logger.exception(msg)
